package org.dromara.system.controller.room;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.dromara.system.domain.bo.FacPageBo;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.FacilitiesVo;
import org.dromara.system.domain.bo.FacilitiesBo;
import org.dromara.system.service.IFacilitiesService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 【请填写功能名称】
 *
 * @author Lion Li
 * @date 2024-12-13
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/facilities")
public class FacilitiesController extends BaseController {

    private final IFacilitiesService facilitiesService;

    /**
     * 查询【请填写功能名称】列表
     */
    @SaCheckPermission("system:facilities:list")
    @GetMapping("/list")
    public TableDataInfo<FacilitiesVo> list(FacilitiesBo bo, PageQuery pageQuery) {
        return facilitiesService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出【请填写功能名称】列表
     */
    @SaCheckPermission("system:facilities:export")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(FacilitiesBo bo, HttpServletResponse response) {
        List<FacilitiesVo> list = facilitiesService.queryList(bo);
        ExcelUtil.exportExcel(list, "【请填写功能名称】", FacilitiesVo.class, response);
    }

    /**
     * 获取【请填写功能名称】详细信息
     *
     * @param facilitiesId 主键
     */
    @SaCheckPermission("system:facilities:query")
    @GetMapping("/{facilitiesId}")
    public R<FacilitiesVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable String facilitiesId) {
        return R.ok(facilitiesService.queryById(facilitiesId));
    }

    /**
     * 新增【请填写功能名称】
     */
    @SaCheckPermission("system:facilities:add")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody FacilitiesBo bo) {
        return toAjax(facilitiesService.insertByBo(bo));
    }

    /**
     * 修改【请填写功能名称】
     */
    @SaCheckPermission("system:facilities:edit")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody FacilitiesBo bo) {
        return toAjax(facilitiesService.updateByBo(bo));
    }

    /**
     * 删除【请填写功能名称】
     *
     * @param facilitiesIds 主键串
     */
    @SaCheckPermission("system:facilities:remove")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.DELETE)
    @DeleteMapping("/{facilitiesIds}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable String[] facilitiesIds) {
        return toAjax(facilitiesService.deleteWithValidByIds(List.of(facilitiesIds), true));
    }

    /**
     * 设施列表查询
     *  ButtonBo
     *  PageQuery
     * @return
     */
    @PostMapping("/facPage")
    @SaCheckPermission("system:facilities:list")
    public TableDataInfo<FacilitiesVo> roomDetail(@Validated(EditGroup.class) @RequestBody FacPageBo bo, PageQuery pageQuery) {
        return facilitiesService.queryPageListByWeixin(bo, pageQuery);
    }

}
